function LandGener
%{
LandGener (C) Maurice.Clerc@WriteMe.com  
Last update: 2017-12-14
---------------------------------------------------------------------------
Generates a landscape, optionally with plateau(s),
to test optimisers (minimisers).

The aim is to build a port-folio of landscapes of different
difficulty levels for minimisation algorithms.

Matlab command to run it:
>> LandGener
It displays a menu with some buttons to push.

You should start playing with 1D and 2D landscapes, and then with
sections of nD landscapes. Have fun!

Suggestions and bug reports are welcome.

---------------------------------------------------------------------------
Two-phase generation
Phase 1
-------
- samples at random N0 points inside the definition space [0,1]^D
- adds the 2^D corners, to be sure to "cover" the whole space
- generates a triangulation, either Delaunay or quicker (but less rigorous)
- optionally selects at random some D-triangles to support plateaus

Phase 2
-------
- assigns a random value to each plateau
- assigns values to all vertices not already assigned
- for each D-triangle, defines the middle, and assigns to it a value
  smaller than the ones of the vertices of this D-triangle

----------
Plots are possible only for dimensions  D=1 and D=2.



---------- Remarks
- Written with Matlab 2017a. Not sure everything works with
    earlier versions

- You can "manually" modify/adding plateaus,
      by pushing the Plateaus button.

- Particular case number_of_vertices=0 (means only the corners of
the search space are used):
- For D=1 it may generates an unimodal landscape
    (sometimes bimodal though).

- For D=2 you may have a bimodal landscape
    or an unimodal domain and a plateau.

- For D>3 you can not directly generate an unimodal landscape
   because the Matlab's Delaunay triangulation does not work
  in that case. But, again, you can "manually" replace all basins but
one by plateaus of same value.

---------- TO DO
- Adding a way to assign user-defined  values to the vertices of
 several basins (i.e. easier than "manually" editing the .txt file)

- more graphical user interfaces
  Example:
  - directly modify the structure in the displayed table
    (see landPlateau and dispStruct)

- Trying to find a safer estimation of the minimum in each domain
  for the difficulty estimation (see diffDelta1)
%}

% Position=[left bottom width height]
b1b2=0.05; % Vertical space text->button
b2b1=0.1; % Button -> text
b2b2=0.1; % Button -> button
w=0.8; % Common width
h=0.06; % Common height

d = figure(...
  'NumberTitle','off',...
  'MenuBar','none',...
  'Units','normalized', ...
  'Position',[0 0.1 0.13 0.8],...
  'Name','Menu');

b2=0.92;
btn = uicontrol('Parent',d,...
  'Units','normalized',...
  'Position',[0.25  b2 0.5 h],...
  'String','New',...
  'BackGroundColor','green',...
  'Callback',@pushNew);
%-------------------------------------------------------------- Examination
b1=b2-b2b1;
txt = uicontrol('Parent',d,...
  'Style','text',...
  'Units','normalized', ...
  'Position',[0.01 b1 w h],...
  'String','--------- Examination --------');
b2=b1-b1b2;

btn = uicontrol('Parent',d,...
  'Units','normalized', ...
  'Position',[0.1 b2 w h],...
  'String','Plot',...
  'Callback',@pushPlot);

b2=b2-b2b2;
btn = uicontrol('Parent',d,...
  'Units','normalized', ...
  'Position',[0.1 b2 w h],...
  'String','Cross section',...
  'Callback',@pushSection);

%-----------------------------------------------------------------  Edition
b1=b2-b2b1;
txt = uicontrol('Parent',d,...
  'Style','text',...
  'Units','normalized', ...
  'Position',[0.01 b1 w h],...
  'String','----------- Edition ----------');

b2=b1-b1b2;

btn = uicontrol('Parent',d,...
  'Units','normalized', ...
  'Position',[0.1 b2 w h],...
  'String','Plateaus',...
  'Callback',@pushPlateau);


%---------------------------------------------------------------- Analysis
b1=b2-b2b1;
txt = uicontrol('Parent',d,...
  'Style','text',...
  'Units','normalized', ...
  'Position',[0.01 b1 w h],...
  'String','----------- Analysis ----------');

b2=b1-b1b2;
btn = uicontrol('Parent',d,...
  'Units','normalized', ...
  'Position',[0.1 b2 w h],...
  'String','Difficulty',...
  'Callback',@pushDifficulty...
  );

b2=b2-b2b2;
btn = uicontrol('Parent',d,...
  'Units','normalized', ...
  'Position',[0.1 b2 w h],...
  'String','NisB evaluation',...
  'Callback',@pushNisB,...
  'Enable','off'...
  );

%--------------------------------------------------------------- Heuristics
b1=b2-b2b1;
txt = uicontrol('Parent',d,...
  'Style','text',...
  'Units','normalized', ...
  'Position',[0.01 b1 w h],...
  'String','----------- Heuristics ----------');

b2=b1-b1b2;
btn = uicontrol('Parent',d,...
  'Units','normalized', ...
  'Position',[0.1 b2 w h],...
  'String','DomByDom',...
  'Callback',@pushLocalSearch...
  );

b2=b2-b2b2;
btn = uicontrol('Parent',d,...
  'Units','normalized', ...
  'Position',[0.1 b2 w h],...
  'String','APS',...
  'Callback',@pushAPS,...
  'Enable','off'...
  );

end
%==========================================================================
% --------------------------------------------------------------------- New
function pushNew(src, event)
  [x,middle,triangle,structure,smooth]=landNew;
end
%--------------------------------------------------------------------- Plot
function pushPlot(src, event)
[x,middle,triangle,structure,smooth]=landRead;
[~,D]=size(x);
if D<=0
  warndlg('No entry, I give up');
end
end
%------------------------------------------------------------ Cross section
function pushSection(src, event)
 landSection;
end
%----------------------------------------------------------------- Plateaus
function pushPlateau(src, event)
  landPlateau;
end
%--------------------------------------------------------------- Difficulty
function pushDifficulty(src, event)
 [x,middle,triangle,structure,smooth]=landRead;
 [~,D]=size(x);
if D<=0
  warndlg('No entry, I give up');
end

quantis=[]; % No discrete problems for the moment
difficulty=diffDelta1(x,middle,triangle,structure,smooth, quantis);
    
end

%--------------------------------------------------------------------  NisB
function puhsNisB(src,event)
end

%------------------------------------------------------------------- Greedy
function pushLocalSearch(src, event)
 [x,middle,triangle,structure,smooth]=landRead;
 [~,D]=size(x);
if D<=0
  warndlg('No entry, I give up');
end

% Note: no quantis for the moment (see greedy)
quantis=[];
%[fBest,domBest,xBest]= greedy (x,middle,triangle, structure,quantis); 
[fBest,domBest,xBest]= DomByDom (x,middle,triangle,...
  structure,smooth,quantis);  


end

%---------------------------------------------------------------------- APS
function puhsAPS(src,event)
end