function [fBest,domBest,xBest]= greedy (x0, middle,triangle, structure,...
  smooth,quantis)
%{
A simple greedy algorithm
  Maurice.Clerc@WriteMe.com
 Version: 2017-12

Here we know the basins and the plateaus. As each basin is a local
unimodal landscape, any not too stupid greedy iterative method
can easily find its minimum.

The present algorithm could (and probably should) be replaced by
a more efficient one, though.

 Outputs
  fBest = best solution (minimum value)
domBest = rank of the domain that contains the minimum
		xBest = best position
   
  %}
  clear valuex
  bigValue=10000;
  
  % Number of domains, and dimension
  [nbDom,D1]=size(triangle);
  D=D1-1;
  
  % Estimates the max number of evaluations
  coeff=2450;
  FEmax=100*floor(coeff*sqrt(D)/100);
  
  % Estimates the population size
  N=floor(90*(1+4*log(D)/1000));
  
  % Number of experiments
  nbExp=10;
    fMini=bigValue*ones(nbDom,nbExp);
  %---------------------------
  h = waitbar(0,'I am looking for the minimum. Please wait...'); %*****
  for nDom=1:nbDom % For each domain ...
    waitbar(nDom/nbDom)
    % The D-triangle that defines the domain
    Dtriangle=triangle(nDom,:);
    
    xTriangle(1:D+1,1:D)=x0(Dtriangle,:);% D+1 points, D coordinates each
 
    if structure(nDom,1)==0 % Plateau. No need of experiments
      fMini(nDom,1)=structure(nDom,2);
      % Arbitrary point on the middle
      xMini(nDom,1:D,1)=mean(xTriangle,1); % Means of each column
      
    else
      
      for nExp=1:nbExp % For real basin, iterative search
        %		Initialises the positions
        for n=1:N
          x(n,1:D)=initInDomain(xTriangle,quantis);
        end
        
        % Initialises the values
        for n=1:N
          f(n)=valuex(x(n,:),x0,middle,triangle, structure,smooth,nDom);
        end
        FEs=N;
        
        % Sort by increasing value. So the best value is f(1)
        [f,Ind]=sort(f);
        x=x(Ind,:);
        
        %---------------------------- Loop
        while FEs<=FEmax
          Ntemp=min(N,FEmax-FEs); 
          
          for n=1:Ntemp
            % Looks around the best
            xTemp(n,:)=around(x(1,1:D),N);
            
            % Checks confinement, and possibly modifies the position;
            xTemp(n,:) = keepInDomain(xTemp(n,:),xTriangle,quantis);
            
            % Quantifies xTemp (if needed)
            if ~isempty(quantis)
              xTemp(n,:)=quantify(xTemp(n,1:D),quantis);
              xTemp(n,:) = keepInDomain(xTemp(n,:),xTriangle,quantis);
            end
            
            % Evaluates
            fTemp(n)=valuex(xTemp(n,:),x0,middle,triangle,...
             structure, smooth,nDom);      
            FEs=FEs+1;
            
            if FEs>=FEmax % Not enough budget for more search, we stop here
              FEs=FEmax+1;
            end    
          end   % for n=1:Ntemp
          
          % Sorts according to the fitness
          [fTemp, Ind]=sort(fTemp);
          xTemp=xTemp(Ind,1:D);         
          
          % Keeps the best ones for the next iteration
          f=fTemp(1:Ntemp);
          x(1:Ntemp,:)=xTemp(1:Ntemp,1:D);
          
        end %  while FEs<=FEmax ----------------------------
        
        % Saves the best result (value and position)
        
        fMini(nDom,nExp)=f(1);
        xMini(nDom,:,nExp)=x(1,:);
      end % for nExp=1:nbExp
      
    end % if structure(nDom,1)==0
    
  end % for nDom=1:nbDom
  close (h) % Close the wait bar
  
  [fBest,Ind]=min(fMini(:));
  [domBest, expBest] = ind2sub(size(fMini),Ind);
  xBest=xMini(domBest,:,expBest);
  
  mess=[num2str(fBest) ' in the domain ' num2str(domBest)];
  h=msgbox(mess,'Probable minimum');
  h.Units='normalized';
  h.Position=[0.35,0.23 0.15 0.06];
  
  fprintf('\n Probable min %f in the domain %i, and on \n ',...
    fBest,domBest);disp(xBest)
  
  clear DA
end % End of greedy
%==========================================================================      
%------------------------------------------------------------- initInDomain
function 	xNew=initInDomain(xTriangle,quantis)
[D1,D]=size(xTriangle);

%		Initialisation  (barycentric method)
% All variables are first supposed to be continuous
  w=rand(D1);
  w=w/sum(w);
  
  xNew(1:D)=w'*xTriangle; 


  % If some variable are discrete, it may be necessary to modify them
if  ~isempty(quantis)
		xNew(:)=quantify(xNew(:),quantis);
end

end
%----------------------------------------------------------------- quantify
function x=quantify(x,quantis)
	D=length(x);
	for d=1:D
		if quantis(d)>0
			x(d)=quantis(d)*round(x(d)/quantis(d));
		end
	end
end

%------------------------------------------------------------- keepInDomain
function x=keepInDomain(x0,xTriangle,quantis)
%	Keeps the position inside the normalised search space
% possibly temporarily expanded
D=length(x0); % Dimension

% Finds the barycentric coordinates
% They should be all negative

X=[x0 1];
onesRow=ones(1,D+1);

%disp(size(x0)); disp(size(xTriangle')); disp(size(X'))

M=[xTriangle';onesRow];

W=linsolve(M,X');

% If necessary re-initialises the position at random
  if min(W)<0
    x=initInDomain(xTriangle,quantis);
    
% fprintf('\n greedy 159, W'); disp(W); 
% fprintf('\n x0'); disp(x0);
 %fprintf('\n x' );disp(x); error(' ')
  else
    x=x0;
  end

end

%------------------------------------------------------------------- around
function xAround=around(x,N)
% Samples a position "around" a given one
	D=length(x);
  sigma=0.5/N; % Empirical
  nSum=5; % 5
  
	for d=1:D 
 		c=bellLike(-0.5,1,nSum); g=sigma*c; % J.  Bell-like perturbation        
  	xAround (d) = min(1,max(0,x(d)+g));
	end
end
%----------------------------------------------------------------- bellLike
function rnd=bellLike(shift, nRnd, nSum)
%	Sample nRnd random numbers from a bell-like distribution
% The higher nSum, the more similar to the Normal distribution
% The support is [shift, 1+shift]
% The mean is shift+0.5

	 rnd=zeros(1,nRnd);  
    for k=1:nSum 
    	rnd=rnd+rand(1,nRnd);
    end
    rnd=shift+rnd/nSum;    
end
