function compareResults (qualityCurve,nbBins)
%{
	 Comparaison of the results of several iterative stochastic algorithms
	 on a given problem
	 2015-09 Maurice.Clerc@WriteMe.com
   Last update: 2017-11-22
	
	 Each experiment has produced a result (one usually keeps the best one)
	 N independent experiments have been launched, with the same "budget"
			(usually a number of evaluations of the objective function)
	 For each algorithm, these N results are saved on a file, one by line
	 The file may contain comments, the data begins at row r0
	
	 WARNING: the number of experiments MUST be the same for all algorithms
	
	 This script analyses the results to produce a fair comparison,
	 based on the their probabilistic distributions, and not only
		as usually, based on (minimum, median, mean(std. dev)).
	
	 That is why N should be big enough to have good estimations of these
		distributions. Typically N>=100.
	
  The quality curve models the user's point of view. From 0 to 3 the user
  is more and more interested in very good (small) results, i.e. is
  more and more demanding.
	 qualityCurve  0 Linear
									1	Parabolic
									2 Normal
									3 Exponential
%}

if ~exist('qualityCurve')
  qualityCurve=3;
end

if ~exist('nbBins')
  nbBins=10;
end

% Where are the data
%	Of course, you have to modify it according to your own files
repertory='./Bests/';
%algo{1}='APS12b-1[9]';
%algo{2}='CMAES-1[9]';
algo{1}='APS12c-1[5]';
algo{2}='TMO-APS-1[5]';
algo{3}='GA-MPC-1[5]';
algo{4}='CMAES-1[5]';

r0=5; % Number of lines to skip (usually comments)
pdfOnly=true;
%-------------------------------------------------------------------------
color={'r','g','b','c','m','y','k'};
nbAlgo=length(algo);

for n=1:nbAlgo
  data{n}=strcat(repertory, algo{n},'.txt');
end

%comparInd=[1 2]; % You may want to compare only some of t
comparInd=[1:nbAlgo];	% If you want to compare all
nbData=length(comparInd);

mini=Inf; maxi=-Inf;

fprintf('\nAlgorithm                  Mini           Median          Mean          Std.dev');

for m=1:nbData
  mm=comparInd(m);
  name{m}=algo{mm};
  results=data{mm};
  
  R{m}=csvread(results,r0,0);
  miniR=min(R{m}); maxiR=max(R{m});
  
  fprintf('\n %15s       %1.5e       %1.5e      %1.5e       %1.5e',...
    name{m},miniR,median(R{m}),mean(R{m}),std(R{m}));
  mini=min(mini,miniR);
  maxi=max(maxi,maxiR);
end

%disp([mini, maxi])  % Common minimum and maximum
score=zeros(nbData,1);
scoreW=score; % Weighted score

% Builds the CDFs (and/or PDFs), and plot them


  if pdfOnly
    fig=figure;  
     hold on
     hl=[]; % Needed to "manually" builds the legend 
            % because a bug in Matlab for multiple plots

    for m=1:nbData
      CDF{m}=CDFplot(R{m},mini,maxi,nbBins,[],[],pdfOnly);
      CDFx=CDF{m}(1,:);
      CDFy=CDF{m}(2,:);
      h=plot(CDFx,CDFy,'LineWidth',2,'DisplayName',name{m},'Color',color{m});
      hl=[hl; h(1)];
    end
 
    legend(hl,name,'Location','east');
    xlabel('Value');
    ylabel('Cumulated probability');
  
  else
    for m=1:nbData
      CDF{m}=CDFplot(R{m},mini,maxi,nbBins,algo{m},[],pdfOnly);
    end
  end

%	Builds the comparison table
% Column	1: the pairs that are compared
%	Column	2: weighted difference
% Column	3: conclusion (better/worse/equivalent)
%	Column	4: confidence coefficient
fprintf('\n\nAlgo                   Algo       Difference')
fprintf('      Conclusion ');

k=0;
for m=1:nbData-1
  for m1=m+1:nbData
    k=k+1;
    
    c{k,1}=[m m1];
    c{k,2}=CDF_compare(CDF{m},CDF{m1},0,qualityCurve);
    
    
    % Evaluates the confidence
    countBetter=0; countWorse=0; countEquiv=0;
    nbBins=length(CDF{1}(1,:));
    
    for nb=1:nbBins
      if CDF{m}(2,nb)>CDF{m1}(2,nb) countBetter=countBetter+1; end
      if CDF{m}(2,nb)<CDF{m1}(2,nb) countWorse=countWorse+1; end
      if CDF{m}(2,nb)==CDF{m1}(2,nb) countEquiv=countEquiv+1; end
    end
    
    countBetter=countBetter/nbBins;
    countWorse=countWorse/nbBins;
    countEquiv=countEquiv/nbBins;
    
    if c{k,2}>0
      if countBetter>0.5
        c{k,3}='better';
        score(m)=score(m)+1;
      else
        c{k,3}='equiv.';
        score(m)=score(m)+1/2;
        score(m1)=score(m1)+1/2;
      end
      
      %	c{k,4}=countBetter;
      
      scoreW(m)=scoreW(m)+c{k,2};
      
    end
    
    if c{k,2}<0
      %if c{k,2}<-0.5
      if countWorse>0.5
        c{k,3}='worse';
        score(m1)=score(m1)+1;
      else
        c{k,3}='equiv.';
        score(m)=score(m)+1/2;
        score(m1)=score(m1)+1/2;
      end
      
      %	c{k,4}=countWorse/nbBins;
      
      scoreW(m1)=scoreW(m1)-c{k,2};
    end
    
    if abs(c{k,2})<realmin
      c{k,3}='equiv.';
      %	c{k,4}=countEquiv/nbBins;
      score(m)=score(m)+1/2;
      score(m1)=score(m1)+1/2;
    end
    
    
    name1=algo{comparInd(c{k,1}(1,1))};
    name2=algo{comparInd(c{k,1}(1,2))};
    
    %fprintf('%20s  vs   %20s   %1.2f    %s   %1.2f',name1,name2,c{k,2},c{k,3},c{k,4})
    fprintf('\n%15s vs %15s   %1.2f         %s',name1,name2,c{k,2},c{k,3})
  end	% % for m1=1:nbData
end % for m=1:nbData

fprintf('\n\n        Algorithm        Score  Weighted_score')
%scoreW=score.*scoreW/nBins;
scoreW=100*scoreW/sum(scoreW);

for m=1:nbData
  mm=comparInd(m);
  fprintf('\n%20s     %2.1f    %2.1f', algo{mm},score(m), scoreW(m));
end
fprintf('\n')
end
